#pragma rtGlobals=1		// Use modern global access method.
#include "mafutils"
#include "mafitc"

// These routines simplify access to the Bruxton SIDX package for controlling cameras such as the Cooke Sensicam.
// User functions:
//	mafCamInit ()
//		Call this first if you want to use these routines
//		Also, creates the mafCamControls panel so you can focus and set exposure times, binning, gains, ROIs, etc.
//
//	mafCamPrepareCamera (howmany, useROI)
//		Sets up a camera to collect howmany images (triggered off a TTL)
//		Optionally, you can set useROI to the ROI number specified in mafCamControls that you want to collect
//		For the whole field, useROI = 0
//		This should be paired with a call to mafCamRetrieveImages or mafCamRetrieveAll
//	mafCamRetrieveImages (howmany, useROI)
//		Retrieves a specified number of images from the camera
//		It needs to be told if these were ROIs (useROI = 1), or the whole field (useROI = 0)
//		This should be preceded by a call to mafCamPrepareCamera
//		If you notice that the progress window is hanging, that means you didn't configure the camera properly.
//		You may need to adjust your triggers and exposure times.  Hit 'Esc' to stop waiting.
//	
//	mafCamPrepareContinuousCamera (useROI)
//		Sets up a camera to collect images as fast as it can.
//		A TTL trigger to the camera is required to start the collection going
//		Optionally, you can set useROI to the ROI number specified in mafCamControls that you want to collect
//		For the whole field, useROI = 0
//		This should be paired with a call to mafCamRetrieveAll
//	mafCamRetrieveAll (howmany, useROI)
//		Retrieves all available images from the camera
//		It needs to be told if these were ROIs (useROI = 1), or the whole field (useROI = 0)
//		This should be preceded by a call to mafCamPrepareCamera or mafCamPrepareContinuousCamera
//
// Other utility functions:
//	mafCamNextShot ()
//		Returns the index for the next shot to be collected
//	mafCamGetExp ()
//		Returns the exposure duration
//	mafCamGetBase (whichROI)
//		Returns the prefix that is used to name the collected images
//	mafCamShow (base, wstart, wend)
//		Makes a fairly nice plot of all images, that are named using the base prefix
//		and indexes that run from wstart to wend.
//		They are put in a window called "Show" + the base prefix
//
// for Image analysis, try using Igor's image analysis package.  Also ImageStats will give you simple statistics.
// I would recommend using "AddToNote" in mafutils to mark each image as to the stimulus and recording conditions.

menu "maf"
	"Init mafCam", mafCamInit()
end

function mafCamInit ()
	if (!datafolderexists ("root:maf"))
		newdatafolder root:maf
	endif
	variable /g root:maf:mafCamCount = 0
	string /g root:maf:mafCamBaseName = "shot"
	variable /g root:maf:sidxHandle, root:maf:mafCamHandle
	variable /g root:maf:mafCamExp, root:maf:mafCamLogExp, root:maf:mafCamDoHist, root:maf:mafCamAutoShow, root:maf:mafCamAutoSave
	variable /g root:maf:mafCamFocussing=0, root:maf:mafCamInterval,root:maf:mafCamHalt, root:maf:mafCamBinning=1
	variable /g root:maf:mafCamLastBin = 1
	variable /g root:maf:mafCamCalibrationX, root:maf:mafCamCalibrationY
	variable /g root:maf:mafCamPrepared=0, root:maf:mafCamShowROI, root:maf:mafCamGain=0, root:maf:mafCamGainMax=0
	variable /g root:maf:mafCamGamma, root:maf:mafCamUseGamma
	NVAR mafCamExp=root:maf:mafCamExp
	setformula mafCamExp, "10^root:maf:mafCamLogExp"
	variable status, cancelled
	string dummyname = ""
	make /u/i/o/n=(1,1) root:maf:mafCamFocus
	make /u/b/o/n=(1,1) root:maf:mafCamROI
	make /o/n=4096 root:maf:mafCamHist
	make /o/n=3 root:maf:mafCamExpTicks={1,2,3}
	make /o/n=3/t root:maf:mafCamExpTickLabels={"0.01 s", "0.1 s", "1 s"}
	make /o/n=4096 root:maf:mafCamGammaFn
	wave mafCamGammaFn=root:maf:mafCamGammaFn
	setformula mafCamGammaFn, "(p/4096)^root:maf:mafCamGamma"
	
	make /o/t/n=(0,3) root:maf:mafCamROIList
	wave /t mafCamROIList=root:maf:mafCamROIList
	setdimlabel 1, 0, ROI, mafCamROIList
	setdimlabel 1, 1, Name, mafCamROIList
	setdimlabel 1, 2, Colour, mafCamROIList
	make /o/n=(0,3,2) root:maf:mafCamROISel
	SetDimLabel 2,1,backColors,root:maf:mafCamROISel 
	make /o/n=(0,4)/i root:maf:mafCamROIRect		// defines ROIs for image in pixels (not m).
	wave mafCamROIRect=root:maf:mafCamROIRect
	setdimlabel 1, 0, top, mafCamROIRect
	setdimlabel 1, 1, left, mafCamROIRect
	setdimlabel 1, 2, bottom, mafCamROIRect
	setdimlabel 1, 3, right, mafCamROIRect
	make /o/n=(1,3) root:maf:mafCamROIColours
	
	// how to avoid double openings?
	mafCamOtherButtonsProc("ReInitButton")
	NVAR mafCamHalt=root:maf:mafCamHalt
	mafCamHalt = 0
	dowindow mafCamControls
	mafLoadSettings ("mafCam", "root:maf:", "mafCamBaseName;mafCamBinning;mafCamGain;mafCamExp;mafCamCalibrationX;mafCamCalibrationY;mafCamShowROI;mafCamDoHist;mafCamGamma;mafCamUseGamma;mafCamAutoSave;mafCamAutoShow")
	if (!v_flag)
		execute "mafCamControls ()"
	endif
	mafCamFixBinning()
	mafCamFixExp()
	mafCamFixGain()
end

function sidxReport (call, status)
	string call
	variable status
	string tempstr
	NVAR sidxHandle=root:maf:sidxHandle, mafCamHalt=root:maf:mafCamHalt
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (status)
		if (strlen (call) == 0)
			call = "unspecified call"
		endif
		sidxgetstatustext sidxHandle, status, tempstr
		print "SIDX error after " + call + ":\r" + tempstr//abort "SIDX error after " + call + ":\r" + tempstr
		abort
	endif
	#endif
end
function cameraReport (call, status)
	string call
	variable status
	string tempstr
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamHalt=root:maf:mafCamHalt
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (status)
		if (strlen (call) == 0)
			call = "unspecified call"
		endif
		sidxcameragetstatustext mafCamHandle, status, tempstr
		print "SIDX error after " + call + ":\r" + tempstr//abort "Camera error after "+ call + ":\r" + tempstr
		abort
	endif
	#endif
end

////////////////////
// For image acquisition, during an experiment
////////////////////

function mafCamPrepareCamera (howmany, useROI)
	// Sets up camera to receive images, either each frame triggered, or just a whole sequence
	variable howmany		// for triggered acquisition of each frame, pass a number (e.g. 6); for continuous acquisition, pass NaN
	variable useROI
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamHalt=root:maf:mafCamHalt, mafCamExp=root:maf:mafCamExp
	NVAR mafCamFocussing=root:maf:mafCamFocussing, mafCamPrepared=root:maf:mafCamPrepared
	wave mafCamROIRect=root:maf:mafCamROIRect
	variable status, i
	
	if (howmany == 0)
		return 0
	endif
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (useROI && useROI > dimsize (mafCamROIRect, 0))
		abort "Illegal ROI specified"
	endif
	if (mafCamFocussing)
		print "Stopping Focus window"
		mafCamGoStopButtonProc ("")
	endif
	if (mafCamPrepared)
		print "Warning: Camera was already prepared!"
		print "Clearing old protocol"
		SIDXAcquisitionAbort mafCamHandle
		SIDXAcquisitionEnd mafCamHandle
	endif
	mafCamFixExp ()	
	mafCamFixBinning ()
	if (numtype (howmany) == 0)
		SIDXTriggerSet mafCamHandle, 2, status	// triggered
	else
		SIDXTriggerSet mafCamHandle, 3, status	// continuous (just trigger sequence)
	endif
	if (status)
		SIDXAcquisitionEnd mafCamHandle
		cameraReport ("PrepareCamera Trigger", status)
	endif
	SIDXROIClear mafCamHandle, status
	if (status)
		SIDXAcquisitionEnd mafCamHandle
		cameraReport ("PrepareCamera ROIClear", status)
	endif
	if (useROI)
		// mafCamROIRect columns = 0=top, 1=left, 2=bottom, 3=right
		SIDXROISet mafCamHandle, mafCamROIRect[useROI-1][1], mafCamROIRect[useROI-1][0], mafCamROIRect[useROI-1][3], mafCamROIRect[useROI-1][2], status
		if (status)
			SIDXAcquisitionEnd mafCamHandle
			cameraReport ("PrepareCamera ROISet", status)
		endif
	endif
	if (numtype (howmany) == 0)	// i.e. number of images is specified
		SIDXAcquisitionBeginSequence mafcamHandle, howmany, status	// triggered (specify how many images)
	else	// i.e. number of images is not specified
		SIDXAcquisitionBeginContinuous mafcamHandle, status
	endif
	if (status)
		SIDXAcquisitionEnd mafCamHandle
		cameraReport ("PrepareCamera Begin", status)
	endif
	SIDXAcquisitionStart mafCamHandle, status
	if (status)
		SIDXAcquisitionEnd mafCamhandle	
		cameraReport ("PrepareCamera Start", status)
	endif
	mafCamPrepared = 1
	#endif
end

function mafCamRetrieveImages (howmany, expTimes, useROI, terminateAcquisition)
	// Gets images from camera, either specified number of frames triggered, or whatever is available
	variable howmany		// specify howmany if you want a certain number of images, or NaN to get whatever is there
	string expTimes	// holds list of exposure times -- if unknown, pass ""
	variable useROI
	variable terminateAcquisition	// set to 1 if you want to stop here
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamCount=root:maf:mafCamCount, mafCamHalt=root:maf:mafCamHalt
	NVAR mafCamPrepared=root:maf:mafCamPrepared, mafCamBinning=root:maf:mafCamBinning, mafCamAutoShow=root:maf:mafCamAutoShow
	NVAR mafCamExp=root:maf:mafCamExp, mafCamGain=root:maf:mafCamGain, mafCamAutoSave=root:maf:mafCamAutoSave
	NVAR mafCamCalibrationX=root:maf:mafCamCalibrationX, mafCamCalibrationY=root:maf:mafCamCalibrationY
	SVAR mafCamBaseName=root:maf:mafCamBaseName
	wave /t mafCamROIList=root:maf:mafCamROIList
	variable status, i, j, dumtime, dumsize, sizex, sizey
	string wname, reportstr=""
	
	exptimes = ReplaceString (";", exptimes, ",")	// make sure it is comma-delimited
	if (howmany == 0)
		return 0
	endif
	if (numtype (howmany) == 0 && itemsinlist (exptimes, ",") > 0 && howmany != itemsinlist (exptimes, ","))
		print "mafCam warning: Number of images collected does not match number of exposure times passed."
	endif
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	if (!mafCamPrepared)
		abort "mafCam Error: Camera not properly prepared for retrieval."
	endif
	
	#if Exists("sidxgetstatustext")
	if (numtype (howmany) == 0)	// number wanted is specified
		do	// wait for the frames to come in
			SIDXAcquisitionFramesAvailable mafcamHandle, i, status
			if (status)
				mafCamPrepared = 0
				SIDXAcquisitionStop mafCamHandle
				SIDXAcquisitionEnd mafCamHandle
				cameraReport ("RetrieveImages FramesAvailable", status)
			endif
			mafprogress (i/howmany)
		while (i < howmany && !(getkeystate(0) & 0x20))	// wait until we get all the frames or 'esc'
		mafprogress (2)
		if (i != howmany)
			mafCamPrepared = 0
			SIDXAcquisitionStop mafCamHandle
			SIDXAcquisitionEnd mafCamHandle
			abort "mafCamRetrieveImages: Only got " + num2str (i) + " out of " + num2str (howmany) + " frames, for some reason."
		endif
		SIDXAcquisitionStop mafCamHandle	// need to stop triggered acquisition before retrieving
	else	// i.e. just get whatever is there
		SIDXAcquisitionFramesAvailable mafcamHandle, howmany, status
		// don't stop a continuous acquisition before retrieving
	endif
	
	if (howmany)
		wname = SelectString(useROI, mafCamBaseName, mafCamROIList[useROI-1][1]) + num2str (mafCamCount)
		SIDXAcquisitionGetFrames mafCamHandle, howmany, 0, $wname, dumtime, status
		if (status)
			mafCamPrepared = 0
			SIDXAcquisitionEnd mafCamHandle
			cameraReport ("RetrieveImages GetFrames", status)
		endif
		AddToNote ($wname, "TIME:" + secs2date (datetime, 0) + " " + secs2time (datetime, 3, 1) + ";EXPOSURE:" +  num2str (mafCamExp) + ";GAIN:" + num2str (mafCamGain))
		if (itemsinlist (exptimes, ",") == howmany)
			AddToNote ($wname, ";EXPTIMES:" + exptimes)
		endif
		if (numtype (mafCamCalibrationX) == 0 && numtype (mafCamCalibrationY) == 0 && mafCamCalibrationX > 0 && mafCamCalibrationY > 0)
			setscale /p x, 0, mafCamBinning/mafCamCalibrationX, "m", $wname
			setscale /p y, 0, mafCamBinning/mafCamCalibrationY, "m", $wname
			AddToNote ($wname, ";CALX:" + num2str (mafCamCalibrationX) + ";CALY:" + num2str (mafCamCalibrationY))
		else
			setscale /p x, 0, mafCamBinning, "pixels", $wname
			setscale /p y, 0, mafCamBinning, "pixels", $wname
		endif
		reportstr += wname + ", "
		mafCamCount += 1
		print "Image stored in " + removeending (reportstr, ", ")
	endif
	if (mafCamAutoSave)
		save /o/p=home $wname
	endif
	if (mafCamAutoShow)
		mafCamShow (SelectString(useROI, mafCamBaseName, mafCamROIList[useROI-1][1]), mafCamCount - howmany, mafCamCount - 1, 1)
	endif
	if (TerminateAcquisition)
		SIDXAcquisitionAbort mafCamhandle
		SIDXAcquisitionEnd mafcamHandle
		mafcamPrepared = 0
	endif
	#endif
	return howmany
end

function /t mafCamDecodeCameraActivity (storeSignal)
	// Returns a list of times that the images were triggered, based on the recorded wave.
	// Useful for Sensicam QE, which puts out a storage signal from the phone jack output.
	// Store signal is high during transfer from camera to board, and low during exposure.
	// The store signal happens after the first exposure, so we have to guess the time of the first.
	// Assumes that there is no gap between the end of storage and the next exposure.
	// You can probably correct for that assumption if you know the actual first exposure time.
	wave storeSignal
	string resultstr = ""
	findlevels /p/q/edge=2 storeSignal, 1000
	wave w_findlevels
	w_findlevels = ceil (w_findlevels)
	if (numpnts (w_findlevels) > 1)
		insertpoints 0, 1, w_findlevels
		w_findlevels[0] = w_findlevels[1] - (w_findlevels[2] - w_findlevels[1])
	endif
	w_findlevels = pnt2x (storeSignal, w_findlevels)
	return (wave2list (w_findlevels, ";"))
end

/////////
// Utilities
///////////

function mafCamFixExp ()
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamExp=root:maf:mafCamExp
	NVAR mafCamHalt=root:maf:mafCamHalt, mafCamFocussing=root:maf:mafCamFocussing
	variable expmin, expmax, expres, status, wasgoing
	wasgoing = mafCamFocussing
	
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to temporarily stop
	endif
	SIDXExposureTimeGetRange mafCamHandle, expmin, expmax, expres, status; cameraReport ("FixExp GetRange", status)
	if (mafCamExp/1000 > expmin && mafCamExp/1000 < expmax)
		SIDXExposureTimeSet mafCamHandle, mafCamExp/1000, status; cameraReport ("FixExp Set", status)
	endif
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to start up again
	endif
	#endif
end

function mafCamFixBinning ()
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamBinning=root:maf:mafCamBinning
	NVAR mafCamHalt=root:maf:mafCamHalt, mafCamFocussing=root:maf:mafCamFocussing
	variable binok, status, wasgoing
	wasgoing = mafCamFocussing
	
	if (mafCamHalt)
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to temporarily stop
	endif
	SIDXBinningXAvailable mafCamHandle, mafCamBinning, binOK, status; cameraReport ("FixBinning XAvailable", status)
	if (!binOK)
		abort "mafCam error: Illegal bin size specified (" + num2str (mafCamBinning) + ")"
	endif
	SIDXBinningYAvailable mafCamHandle, mafCamBinning, binOK, status; cameraReport ("FixBinning YAvailable", status)
	if (!binOK)
		abort "mafCam error: Illegal bin size specified (" + num2str (mafCamBinning) + ")"
	endif
	SIDXBinningSet mafCamHandle, mafCamBinning, mafCamBinning, status; cameraReport ("FixBinning Set", status)
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to start up again
	endif
	#endif
end

function mafCamFixGain ()
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamGain=root:maf:mafCamGain, mafCamGainMax=root:maf:mafCamGainMax
	NVAR mafCamHalt=root:maf:mafCamHalt, mafCamFocussing=root:maf:mafCamFocussing
	variable status, wasgoing

	wasgoing = mafCamFocussing
	if (mafCamHalt)
		abort "mafCam error: Can't set camera gain without initialized camera."
	endif
	#if Exists("sidxgetstatustext")
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to temporarily stop
	endif
	if (mafCamGain > mafCamGainMax)
		mafCamGain = mafCamGainMax
	endif
	SIDXGainSet mafCamHandle, mafCamGain, status; cameraReport ("Fix Gain", status)
	if (wasGoing)	
		mafCamGoStopButtonProc("") // need to start up again
	endif
	#endif
end

function mafCamShow (base, istart, iend, autoscale)
	string base
	variable istart, iend, autoscale
	variable mymin = inf, mymax = -inf
	variable i, width, spacer = .02
	
	dowindow $("Show" + base)
	if (v_flag)	//it is already there, so remove all contents
		GetWindow $("Show" + base) wavelist
		wave /t w_wavelist
		 for (i = dimsize (w_wavelist, 0) - 1; i >= 0 ; i -= 1)			// need to work backwards in case someone is there twice
		 	RemoveImage /w=$("Show" + base) $(w_wavelist[i][1] )
		 endfor
	else
		display
		dowindow /c $("Show" + base)
		ModifyGraph margin=14
	endif
	width = (1 - (iend - istart) * spacer) / (iend - istart + 1)
	if (autoscale)
		for (i = istart; i <= iend; i += 1)
			imagestats /m=1 $(base + num2str (i))
			mymin = min (mymin, v_min)
			mymax = max (mymax, v_max)
		endfor
	endif
	for (i = istart; i <= iend; i += 1)
		appendimage /w=$("Show" + base)/l=$("l" + num2str (i))/t=$("t"+num2str(i)) $(base + num2str (i))
		if (i == istart)
			modifygraph /w=$("Show" + base) width={Plan,1,$("t"+num2str(i)),$("l"+num2str(i))}
		endif
		if (autoscale)
			modifyimage /w=$("Show" + base) $(base + num2str (i)) ctab={mymin, mymax, $(popupval ("colortablepopup", "mafCamControls")), 0}
		endif
		SetAxis/w=$("Show" + base)/A/R $("l" + num2str (i))
		modifygraph /w=$("Show" + base) axisenab($("t"+num2str(i)))={(i - istart) * (width + spacer), (i - istart) * (width + spacer) + width}
	endfor
	ModifyGraph /w=$("Show" + base) mirror=2, nticks=4, minor=1, fSize=8, standoff=0, btLen=3, tlOffset=-2, freePos=0
end

function mafCamDoDFF (w, imZeroes, dest, furalike, doDF)
	// returns the DF or DF/F of an image stack in w
	// images before stimulation are called "zeroes"
	// if there are multiple zero images, they are averaged together before calculating the DF or DF/F
	wave w
	string dest
	variable imZeroes
	variable furalike, doDF
	variable i, scaler
	variable thisx, thisy

	if (imZeroes == 0)
		abort "mafCamDoDFF error: No zero images specified"
	endif
	// need to take care of image registration
	make /o /n=(dimsize (w, 0), dimsize (w, 1)) /i mafCamTempZero
	copyscales /p w mafCamTempZero
	mafCamTempZero = 0
	for (i = 0; i < imZeroes; i += 1)
		thisx = NumberByKeySafe ("REGX" + num2str (i), note (w), 0)
		thisy = NumberByKeySafe ("REGY" + num2str (i), note (w), 0)
		mafCamTempZero += w[p + thisy][q + thisx][i]
//		if (thisx < 0)
	endfor
	mafCamTempZero /= imZeroes
	
	duplicate /o w $dest
	wave wd=$dest
	if (doDF)
		redimension /i wd
	else
		redimension /s wd
	endif

	if (doDF)
		if (furalike)
			wd = mafCamTempZero[p][q] - w
		else
			wd = w - mafCamTempZero[p][q]
		endif
	else
		if (furalike)
			wd  = (1 - w/mafCamTempZero[p][q]) * 100
		else
			wd = (w/mafCamTempZero[p][q] - 1) * 100
		endif
	endif
end

function mafCamNextShot ()
	NVAR mafCamCount=root:maf:mafCamCount
	return (mafCamCount)
end

function mafCamGetExp ()
	NVAR mafCamExp=root:maf:mafCamExp
	return (mafCamExp)
end

function /t mafCamLastImage (whichROI)
	variable whichROI
	return mafCamGetBase (whichROI) + num2str (mafCamNextShot () - 1)
end

function /t mafCamGetBase (whichROI)
	variable whichROI
	SVAR mafCamBaseName=root:maf:mafCamBaseName
	wave /t mafCamROIList=root:maf:mafCamROIList
	if (whichROI == 0)
		return mafCamBaseName
	elseif (whichROI <= dimsize (mafCamROIList, 0))
		return (mafCamROIList[whichROI - 1][1])
	else
		abort "Illegal ROI number specified"
	endif
end

///////////////
// mafCamControls Interface
///////////////
Function mafCamExpField(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName, varStr, varName
	Variable varNum
	NVAR mafcamlogexp=root:maf:mafcamlogexp
	mafcamlogexp = log(varnum)
	mafCamFixExp()
End
Function mafCamExpSliderProc(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue,event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved
	NVAR mafCamExp=root:maf:mafCamExp, mafCamFocussing=root:maf:mafCamFocussing

	// change exposure if slider is moved, unless the auto shutter is on, in which case, wait until mouse up
	if (!mafCamFocussing || (event %& 0x8 && !checkval ("ShutterCheck", "mafCamControls")) || (event %& 0x4 && checkval ("ShutterCheck", "mafCamControls")))
		mafCamFixExp ()
	endif
	return 0
End

Window mafCamControls() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(553,75,879,355)
	Slider Exposureslider1,pos={267,192},size={50,55},fSize=9
	Slider Exposureslider1,limits={0,2,0},variable= root:maf:mafCamGamma,side= 2,ticks= 2
	GroupBox group1,pos={104,8},size={82,59},title="pixels/m"
	ListBox list0,pos={8,180},size={175,91},proc=mafCamListBoxProc
	ListBox list0,listWave=root:maf:mafCamROIList,selWave=root:maf:mafCamROISel
	ListBox list0,colorWave=root:maf:mafCamROIColours,mode= 5,widths={36,84,37}
	ListBox list0,userColumnResize= 1
	Button mafCamGoStopbutton,pos={267,24},size={50,20},proc=mafCamGoStopButtonProc,title="Focus"
	Slider Exposureslider,pos={12,76},size={165,57},proc=mafCamExpSliderProc,fSize=9
	Slider Exposureslider,limits={1,3,0},variable= root:maf:mafCamLogExp,side= 2,vert= 0
	Slider Exposureslider,userTicks={:maf:mafCamExpTicks,:maf:mafCamExpTickLabels}
	SetVariable setvar4,pos={177,108},size={50,16},proc=mafCamExpField,title=" "
	SetVariable setvar4,format="%0.3G ms",limits={0,inf,0},value= root:maf:mafCamExp
	SetVariable setvar5,pos={7,6},size={78,16},title="prefix"
	SetVariable setvar5,limits={0,inf,0},value= root:maf:mafCamBaseName
	CheckBox Histcheck,pos={239,120},size={65,14},proc=AttachHistogramCheckProc,title="Histogram"
	CheckBox Histcheck,variable= root:maf:mafCamDoHist
	CheckBox AutoScalecheck,pos={239,102},size={67,14},proc=AutoScaleCheckProc,title="AutoScale"
	CheckBox AutoScalecheck,value= 1
	Button Releasebutton,pos={259,45},size={58,20},proc=mafCamOtherButtonsProc,title="Release"
	Button ReInitbutton,pos={205,45},size={50,20},proc=mafCamOtherButtonsProc,title="ReInit"
	CheckBox ROIcheck,pos={187,186},size={72,14},proc=ShowROICheckProc,title="Show ROIs"
	CheckBox ROIcheck,variable= root:maf:mafCamShowROI
	SetVariable setvar6,pos={12,32},size={85,16},proc=BinningSetVarProc,title="Binning"
	SetVariable setvar6,limits={0,inf,1},value= root:maf:mafCamBinning,noedit= 1
	Button DeleteROIButton,pos={188,228},size={50,20},proc=ROIButtonProc,title="Delete"
	Button ClearAllROIButton,pos={189,249},size={50,20},proc=ROIButtonProc,title="Clear All"
	GroupBox group0,pos={191,8},size={132,60},title="Camera"
	SetVariable setvar7_1,pos={28,52},size={69,16},proc=mafCamSetGainProc,title="Gain"
	SetVariable setvar7_1,limits={0,inf,1},value= root:maf:mafCamGain,noedit= 1
	CheckBox Displaycheck,pos={11,147},size={75,14},title="Auto display"
	CheckBox Displaycheck,variable= root:maf:mafCamAutoShow
	PopupMenu ColorTablePopup,pos={97,144},size={200,24}
	PopupMenu ColorTablePopup,mode=6,popvalue="",value= #"\"*COLORTABLEPOP*\""
	SetVariable setvar7,pos={128,24},size={41,16},title="X"
	SetVariable setvar7,limits={-inf,inf,0},value= root:maf:mafCamCalibrationX
	SetVariable setvar8,pos={128,43},size={41,16},title="Y"
	SetVariable setvar8,limits={-inf,inf,0},value= root:maf:mafCamCalibrationY
	Button CaptureButton,pos={204,23},size={50,20},proc=mafCamOtherButtonsProc,title="Capture"
	Button AddROIButton,pos={188,206},size={50,20},proc=ROIButtonProc,title="Add"
	CheckBox Shuttercheck,pos={247,72},size={75,14},title="Auto shutter",value= 0
	Button SaveButton,pos={243,249},size={79,20},proc=mafCamOtherButtonsProc,title="Save Settings"
	CheckBox GammaCheck,pos={280,174},size={26,15},proc=AutoScaleCheckProc,title=" \\F'Symbol'g"
	CheckBox GammaCheck,variable= root:maf:mafCamUseGamma
EndMacro

Function AutoScaleCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	dowindow mafCam
	if (v_flag == 0)
		return 0
	endif
	if (checkval ("AutoScaleCheck", "mafCamControls"))
		ModifyImage /w=mafCam mafCamFocus ctab= {*,*,Grays,0}
	else
		modifyimage /w=mafCam mafCamFocus ctab= {0,4094,Grays,0}
		ModifyImage /w=mafCam mafCamFocus minRGB=(0,0,65535),maxRGB=(65535,0,0)
	endif
	if (checkval ("GammaCheck", "mafCamControls"))
		ModifyImage /w=mafCam mafCamFocus lookup= mafCamGammaFn// ctabAutoscale=0,
	else
		ModifyImage /w=mafCam mafCamFocus lookup= $("")	//ctabAutoscale=0,
	endif
End

End

function mafCamUpdateHistogram ()
	wave mafCamROI=root:maf:mafCamROI
	wave mafCamFocus=root:maf:mafCamFocus
	NVAR mafCamShowROI=root:maf:mafCamShowROI, mafCamDoHist=root:maf:mafCamDoHist
	
	if (!mafCamDoHist)
		return 0
	endif
	if (mafCamShowROI)
		imagehistogram /r=mafCamROI mafCamFocus
	else
		imagehistogram mafCamFocus
	endif
	duplicate /o w_imagehist root:maf:mafCamHist
	// fix axes on histogram
	SetAxis /W=mafCamHistogram /A left
	SetAxis /W=mafCamHistogram /A bottom
end

Function AttachHistogramCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	dowindow mafCam
	if (v_flag == 0)
		return 0
	endif
	
	dowindow mafCamHistogram
	if (v_flag == 0 && checked)	// no window, but now checked, so create window
		Display /N=mafCamHistogram/W=(30,278.75,291,345.5)/K=2 root:maf:mafCamHist as "Histogram"
		ModifyGraph margin(left)=7,margin(bottom)=18,margin(top)=7,margin(right)=7
		ModifyGraph log(left)=1,tick(left)=3, noLabel(left)=2
		ModifyGraph fSize(bottom)=8
		ModifyGraph standoff(left)=0,axThick(left)=0
		ModifyGraph btLen(bottom)=4,stLen(bottom)=2
		ModifyGraph live=1
		AutoPositionWindow/M=1/R=mafCam mafCamHistogram
		mafCamUpdateHistogram()
	elseif (v_flag == 1 && !checked)	// yes window, but now unchecked, so get rid of it
		KillWindow mafCamHistogram 
	endif
End
function Camfindselectedrow ()
	wave mafCamROISel=root:maf:mafCamROISel
	variable i, j
	for (i = 0; i < dimsize (mafCamROISel, 0); i += 1)
		for (j = 0; j < dimsize (mafCamROISel, 1); j += 1)
			if (mafCamROISel[i][j] %& 0x01)	// i.e. selected bit is set
				return i
			endif
		endfor
	endfor
	return -1
end
menu "mafCamColourPopupMenu", contextualmenu
	"*COLORPOP*", ;
end

Function ROIButtonProc(ctrlName) : ButtonControl
	String ctrlName
	NVAR mafCamCalibrationX=root:maf:mafCamCalibrationX, mafCamCalibrationY=root:maf:mafCamCalibrationY
	NVAR mafCamShowROI=root:maf:mafCamShowROI
	wave /t mafCamROIList=root:maf:mafCamROIList
	wave mafCamROISel=root:maf:mafCamROISel, mafCamROIRect=root:maf:mafCamROIRect
	wave mafCamROIColours=root:maf:mafCamROIColours
	variable ROICount, selRow

	dowindow mafCam
	if (v_flag == 0)
		abort "Select ROI in Focus window using marquee before hitting Add button."
	endif

	strswitch (ctrlName)
		case "AddROIButton":
			GetMarquee /K/W=mafCam top, left
			if (v_flag == 0)
				abort "Select ROI in Focus window using marquee before hitting Add button."
			endif
			ROICount = dimsize (mafCamROIList, 0)
			redimension /n=(ROICount + 1, 3) mafCamROIList
			redimension /n=(ROICount + 1, 3, 2) mafCamROISel
			redimension /n=(ROICount + 2, 3) mafCamROIColours
			redimension /n=(ROICount + 1, 4) mafCamROIRect
			mafCamROIList[ROICount][1]="ROI" + num2str (ROICount + 1) + "_"
			
			mafCamROIRect[ROICount][0] = v_top * (mafCamCalibrationY != 0 ? mafCamCalibrationY : 1)	//***fix by calibration
			mafCamROIRect[ROICount][1] = v_left * (mafCamCalibrationX != 0 ? mafCamCalibrationX : 1)
			mafCamROIRect[ROICount][2] = v_bottom * (mafCamCalibrationY != 0 ? mafCamCalibrationY : 1)
			mafCamROIRect[ROICount][3] = v_right * (mafCamCalibrationX != 0 ? mafCamCalibrationX : 1)
			break
		case "DeleteROIButton":
			selrow = Camfindselectedrow ()
			if (selrow > -1)
				DeletePoints /M=0 selrow, 1, mafCamROIList
				DeletePoints /M=0 selrow, 1, mafCamROISel
				DeletePoints /M=0 selrow, 1, mafCamROIRect
				DeletePoints /M=0 selrow+1, 1, mafCamROIColours
			endif
			break
		case "ClearAllROIButton":
			redimension /n=(0,3) mafCamROIList
			redimension /n=(0,3,2) mafCamROISel
			redimension /n=(0,3) mafCamROIColours
			redimension /n=(0,4) mafCamROIRect
			break
	endswitch
	mafCamROIList[][0] = num2str (p + 1)
	mafCamROISel[][0][0] = 0
	mafCamROISel[][1][0] = 2
	mafCamROISel[][2][1] = p + 1
	button DeleteROIButton disable=(dimsize (mafCamROIList, 0) == 0 ? 2 : 0), win=mafCamControls
	
	// now update display
	mafCamUpdateROIRects ()
//	if (mafCamShowROI)
//		imagegenerateroimask /w=mafcam/e=1/i=0 mafcamfocus
//		duplicate /o m_roimask root:maf:mafCamROI
//	endif
	mafCamUpdateHistogram ()
End

Function mafCamListBoxProc(lba) : ListBoxControl
	STRUCT WMListboxAction &lba
	wave /t mafCamROIList=root:maf:mafCamROIList
	wave mafCamROIColours=root:maf:mafCamROIColours
	Variable row = lba.row
	Variable col = lba.col

	if (lba.eventCode == 1 && row < numpnts (mafCamROIList) && (lba.eventMod & 0x10) && col == 2)
		popupcontextualmenu /n "mafCamColourPopupMenu"
		if (v_flag != -1)	// i.e something chosen
			mafCamROIColours[row+1][0]=v_red
			mafCamROIColours[row+1][1]=v_green
			mafCamROIColours[row+1][2]=v_blue
			mafcamupdateroirects()
		endif
	endif
	return 0
End

Function mafCamOtherButtonsProc(ctrlName) : ButtonControl
	String ctrlName
	NVAR mafCamHandle=root:maf:mafCamHandle, sidxhandle=root:maf:sidxhandle, mafCamHalt=root:maf:mafCamHalt, mafCamGainMax=root:maf:mafCamGainMax
	SVAR mafCamBaseName=root:maf:mafCamBaseName
	NVAR mafCamCount=root:maf:mafCamCount
	variable status, cancelled
	
	#if Exists("sidxgetstatustext")
	strswitch (ctrlName)
		case "ReleaseButton":
			SIDXCameraClose mafCamHandle
			SIDXClose sidxHandle
			mafCamHalt = 1
			break
		case "ReInitButton":
			mafCamHalt = 0
			SIDXOpen sidxHandle, status; sidxReport ("Init Open", status)
			SIDXDialogCameraSelector sidxhandle, cancelled, mafCamHandle, status; sidxReport ("Init Select", status)
			if (cancelled)
				mafCamHalt = 1
				break
			else
				SIDXGainGetCount mafCamHandle, mafCamGainMax, status
				mafCamGainMax -= 1
				if (status)
					cameraReport ("ReInitButton Get Gain", status)
					mafCamGainMax = 0
				endif
			endif
			break
		case "CaptureButton":
			duplicate root:maf:mafCamFocus $(mafCamBaseName + num2str (mafCamCount))
			print "Focus image captured as " + mafCamBaseName + num2str (mafCamCount)
			mafCamCount += 1
			break
		case "SaveButton":
			mafSaveSettings ("mafCam", "root:maf:", "mafCamBaseName;mafCamBinning;mafCamGain;mafCamExp;mafCamCalibrationX;mafCamCalibrationY;mafCamShowROI;mafCamDoHist;mafCamGamma;mafCamUseGamma;mafCamAutoShow;mafCamAutoSave")
			break
	endswitch
	#endif
End

Function BinningSetVarProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName, varStr, varName
	Variable varNum
	NVAR mafCamBinning=root:maf:mafCamBinning, mafCamLastBin=root:maf:mafCamLastBin
	
	if (mafCamBinning > mafCamLastBin)	// i.e. incremented
		mafCamBinning = mafCamLastBin * 2
	elseif (mafCamBinning < mafCamLastBin && mafCamLastBin > 1)	// i.e. decremented
		mafCamBinning = mafCamLastBin / 2
	endif
	mafCamBinning = 2^(round (log (mafCamBinning) / log (2)))
	mafCamLastBin = mafCamBinning
	mafCamFixBinning ()
End

Function ShowROICheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	mafCamUpdateROIRects ()
	mafCamUpdateHistogram ()
End

Function mafCamSetGainProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName, varStr, varName
	Variable varNum
	mafCamFixGain ()
End

////////////////
// Focussing routines
////////////////

Function mafCamGoStopButtonProc(ctrlName) : ButtonControl
	String ctrlName
	NVAR mafCamFocussing=root:maf:mafCamFocussing, mafCamInterval=root:maf:mafCamInterval, mafCamHalt=root:maf:mafCamHalt
	NVAR mafCamExp=root:maf:mafCamExp, mafCamBinning=root:maf:mafCamBinning
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamUseGamma=root:maf:mafCamUseGamma
	variable status, dumsize, xsize, ysize

	if (mafCamHalt)
		CtrlNamedBackground mafCamBack, stop
		abort "Can't do anything until a camera is successfully initialized"
	endif
	#if Exists("sidxgetstatustext")
	if (mafCamFocussing)
		SIDXAcquisitionStop mafCamHandle
		SIDXAcquisitionEnd mafCamHandle
		CtrlNamedBackground mafCamBack, stop
		Button mafCamGoStopButton title="Focus", win=mafCamControls
		mafCamFocussing = 0
		if (checkval ("ShutterCheck", "mafCamControls"))
			mafITC_SetTTL (3, 0)		// close shutter
		endif
	else
		dowindow /f mafCam
		if (!v_flag)
			newImage /k=1 root:maf:mafCamFocus
			dowindow /c mafCam
			ModifyGraph width={Plan,1,top,left}
			AutoScaleCheckProc ("", 0)
		endif

		SIDXExposureTimeSet mafCamHandle, mafCamExp/1000, status; cameraReport ("GoStop Exp", status)
		SIDXBinningSet mafCamHandle, mafCamBinning, mafCamBinning, status; cameraReport ("FixBinning Set", status)
		SIDXTriggerSet mafCamHandle, 0, status; cameraReport ("GoStop Trigger", status)
		SIDXROIClear mafCamHandle, status; cameraReport ("GoStop ROI", status)
		SIDXAcquisitionBeginFocus mafCamHandle, status; cameraReport ("GoStop Begin", status)
		SIDXAcquisitionStart mafCamHandle, status; cameraReport ("GoStop Start", status)
		CtrlNamedBackground mafCamBack, start, period = 0, proc=mafCamDoSweep
		Button mafCamGoStopButton title="Stop", win=mafCamControls
		mafCamFocussing = 1
		if (checkval ("ShutterCheck", "mafCamControls"))
			mafITC_SetTTL (3, 1)		// open shutter
		endif
	endif
	#endif
End

function mafCamDoSweep (s)
	STRUCT WMBackgroundStruct &s
	NVAR mafCamHandle=root:maf:mafCamHandle, mafCamHalt=root:maf:mafCamHalt, mafCamDoHist=root:maf:mafCamDoHist
	NVAR mafCamShowROI=root:maf:mafCamShowROI, mafCamBinning=root:maf:mafCamBinning
	NVAR mafCamCalibrationX=root:maf:mafCamCalibrationX, mafCamCalibrationY=root:maf:mafCamCalibrationY
	variable status=0, numframes=0, dumtime
	
	if (mafCamHalt)
		return 0
	endif
	#if Exists("sidxgetstatustext")
	SIDXAcquisitionFramesAvailable mafCamhandle, numframes, status
	if (status)
		print "Frames available error: status is ", status
		return 1
	endif
	if (numframes > 0)
		SIDXAcquisitionGetFrames mafCamHandle, 1, 0, root:maf:mafCamFocus, dumtime, status
		wave mafCamFocus=root:maf:mafCamFocus
		if (numtype (mafCamCalibrationX) == 0 && numtype (mafCamCalibrationY) == 0 && mafCamCalibrationX > 0 && mafCamCalibrationY > 0)
			setscale /p x, 0, mafCamBinning/mafCamCalibrationX, "m", mafCamFocus
			setscale /p y, 0, mafCamBinning/mafCamCalibrationY, "m", mafCamFocus
		else
			setscale /p x, 0, mafCamBinning, "pixels", mafCamFocus
			setscale /p y, 0, mafCamBinning, "pixels", mafCamFocus
		endif		
		if (status)
			print "GetFrames error: status is ", status
			return 1
		endif
		mafCamUpdateHistogram ()
	endif
	#endif
	return 0
end
function mafCamUpdateROIRects ()
	variable i, j
	wave mafCamROIRect=root:maf:mafCamROIRect, mafCamROIColours=root:maf:mafCamROIColours
	NVAR mafCamShowROI=root:maf:mafCamShowROI

	dowindow mafCam
	if (v_flag == 0)
		return 0
	endif
	SetDrawLayer /K /w=mafCam ProgFront
	if (!mafCamShowROI)
		return 0
	endif
	SetDrawLayer /W=mafCam ProgFront
	for (j = 0; j < DimSize (mafCamROIRect, 0); j += 1)
		SetDrawEnv /W=mafCam xcoord=top,fillpat=0,ycoord=left,linefgc= (mafCamROIColours[j+1][0],mafCamROIColours[j+1][1],mafCamROIColours[j+1][2])
		// if (scaling?)
		// else
			DrawRect /W=mafCam mafCamROIRect[j][1], mafCamROIRect[j][0], mafCamROIRect[j][3], mafCamROIRect[j][2]
		// endif
	endfor
end
